==> ciao-4.6-src-core.tar.gz: ciao-4.6/src/lib/help/XMLFParser.cc <==
// 
//  Copyright (C) 2002,2005,2007,2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//




// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include "cxcconfig.h"
#include "XMLFParser.hh"




int XMLFParser::globalEnvInitFlag = 0;
int XMLFParser::instanceCount = 0;
SAX2XMLReader* XMLFParser::parser;
XMLFormatter* XMLFParser::fFormatter;

// ---------------------------------------------------------------------------
//  Name: XMLFParser::XMLFParser 
// 
//  Describtion: Constructor for XMLFParser
//
//  Pre:  
//        
//
//  Post: Throws int exception if init error occured
//        Because of global data in sax2 parser number of XMLFParser instances
//        is maintained
//        
//
//
// ---------------------------------------------------------------------------

XMLFParser::XMLFParser() throw(int)
{
  instanceCount++;
}


// ---------------------------------------------------------------------------
//  Name: XMLFParser::~XMLFParser 
// 
//  Describtion: Destructor for XMLFParser
//
//  Pre:  
//        
//
//  Post: Because of global data in sax2 parser number of XMLFParser instances
//        is maintained
//        
//
//
// ---------------------------------------------------------------------------

XMLFParser::~XMLFParser()
{

  --instanceCount;

  if (( !instanceCount) && ( globalEnvInitFlag) )
    {
      delete parser;
      delete fFormatter; 
      XMLPlatformUtils::Terminate();
      globalEnvInitFlag=0;

    }

}


// ---------------------------------------------------------------------------
//  Name: XMLFParser::parse 
// 
//  Describtion: the only entry point to the parser
//
//  Pre:  inFilePath - xml file to be parsed
//        inDTDPath  - location of dtd, if set to non empty string it overrides
//                     dtd specified in xml files
//        inValidateFlag - true if validation of xml file for complience with 
//                           dtd should be performed
//  Post: outData - formatted content of xml file
//        on error outData contains error message if any
//        Return values: 1 on success, 0 otherwise
//
//
// ---------------------------------------------------------------------------

int XMLFParser::parse(const AhelpStrings& inFilePath,const AhelpStrings& inDTDPath,bool inValidateFlag,DTDElement& outElement)
{

  int status = 1;
  char* tmpPtr;
  top=&outElement;
  endPtr = top;

  top->Reset();


  // Make sure xax2 environment is initialized only once
  
  if ( !globalEnvInitFlag )
    {
      try
	{
	  XMLPlatformUtils::Initialize();
	  parser = XMLReaderFactory::createXMLReader();
	  
	  // fFormatter is used by sax2 parser to convert text from internal
	  // representation to specified format 
	  
	  fFormatter = new XMLFormatter("LATIN1", this, XMLFormatter::NoEscapes,XMLFormatter::UnRep_CharRef);
	}
      catch (const XMLException& e)
	{
	  outElement.data="XMLFParser::XMLFParser  Error during initialization! :";
	  outElement.data.append("Exception message is:  ");
	  tmpPtr=XMLString::transcode(e.getMessage());
	  outElement.data.append(tmpPtr);
	  XMLString::release(&tmpPtr);
	  status = 0;
	}
      globalEnvInitFlag=1;
    };
  
  if ( status )
    {
      
      try
	{
	  if ( inDTDPath == "")
	    {
	      DTDSource = 0;
	    }
	  else
	    {
	      XMLCh* tmpXMLPtr =  XMLString::transcode(inDTDPath.chars());
	      DTDSource = new LocalFileInputSource(tmpXMLPtr);
	      XMLString::release(&tmpXMLPtr);
	    };
	  
	  if ( inValidateFlag )
	    {      
	      parser->setFeature(XMLUni::fgSAX2CoreValidation, true);
	      parser->setFeature(XMLUni::fgXercesDynamic, false);
	    }
	  else
	    {
	      parser->setFeature(XMLUni::fgSAX2CoreValidation, false);
	    };
	  
	  
	  // sax2 needs handlers for various aspects of parsing
	  // see sax2 documentation in xerces package for more
	  
	  parser->setContentHandler(this);
	  parser->setErrorHandler(this);
	  parser->setEntityResolver(this);
	  
	  parser->parse(inFilePath.chars());

	}
      catch (const XMLException& e)
	{
	  outElement.data = "Error in XMLFParser::parse while parsing '";
	  outElement.data.append(inFilePath.chars() );
	  outElement.data.append("'\n");
	  outElement.data.append("Exception message is:  \n");
	  tmpPtr= XMLString::transcode(e.getMessage());
	  outElement.data.append(tmpPtr);
	  XMLString::release(&tmpPtr);
	  status = 0;
	}
      catch(...)
	{
	  status = 0;
	};
    };


  if ( ! status )
    {
      outElement.children.RemoveAll();
    };

  return status;
};


// ---------------------------------------------------------------------------
//  XMLFParser: Overrides of the output formatter target interface
//              See xerces docs for more info
// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------
//  Name: XMLFParser::writeChars 
// 
//  Describtion: implements handling of converted raw data
//
//  Pre:  
//        
//
//  Post: Adds new raw data element to the parse tree
//        
//        
//
//
// ---------------------------------------------------------------------------



#ifdef __XERCES_2_8__

void XMLFParser::writeChars(const XMLByte* const toWrite,
			    const unsigned int count,
			    XMLFormatter* const formatter)
  
#else
  
void XMLFParser::writeChars(const   XMLByte* const  toWrite, 
			    XMLSize_t    count, 
			    XMLFormatter* const   formatter)

#endif
{

  //Handle new raw data 

  DTDElement newDTDElement;
  int indx=0;
  int mergeFlag=0;
  char *buf=0;

  //Create blockText out of it


  if ( count > 0 )
    {
      buf = new char[count+1];
      strncpy(buf,(char*)toWrite,count);
      buf[count]=0;
    };


  // Try to merge it with previous raw data block
  if ( top->CanCompact(endPtr->name) && ( count==0))
    {
      mergeFlag = 1;
    }
  else
    {
      
      if ( endPtr->children.GetLength())
	{
	  
	  if ( endPtr->children.GetLast().name == DTDElement::TOK_PCDATA )
	    {
	      endPtr->children.GetLast().data.append(buf);
	      mergeFlag = 1;
	    };
	};
    };
  
  if ( ! mergeFlag )
    {
      newDTDElement.data = buf;
      newDTDElement.name = DTDElement::TOK_PCDATA;
      
      endPtr->children.Append(newDTDElement);
      endPtr->children.GetLast().parent = endPtr;
    };

  if ( buf)
    {
      delete[] buf;
    };

}


// ---------------------------------------------------------------------------
//  XMLFParser: Overrides of the SAX ErrorHandler interface
//              See xerces docs for more info
// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------
//  Name: XMLFParser::error
// 
//  Describtion: implements handling of error exeptions
//
//  Pre:  See xerces docs
//        
//
//  Post: See xerces docs 
//        
//        
//
//
// ---------------------------------------------------------------------------


void XMLFParser::error(const SAXParseException& e) throw(int)
{
  char* fileName = XMLString::transcode(e.getSystemId());
  char* message=XMLString::transcode(e.getMessage());
  formatErrorMsg("Error at file : ",
		 fileName,
		 e.getLineNumber(),
		 e.getColumnNumber(),
		 message,
		 top->data);

  XMLString::release(&fileName);
  XMLString::release(&message);
  //Propagate exception higher up in the calling stack

  throw 0;
}



// ---------------------------------------------------------------------------
//  Name: XMLFParser::error
// 
//  Describtion: implements handling of ftal error exeptions
//
//  Pre:  See xerces docs
//        
//
//  Post: See xerces docs 
//        
//        
//
//
// ---------------------------------------------------------------------------



void XMLFParser::fatalError(const SAXParseException& e) throw(int)
{
  char* fileName=XMLString::transcode(e.getSystemId());
  char* message=XMLString::transcode(e.getMessage());
  formatErrorMsg("Fatal Error at file : ",
		 fileName,
		 e.getLineNumber(),
		 e.getColumnNumber(),
		 message,
		 top->data);

  //Propagate exception higher up in the calling stack
  XMLString::release(&fileName);
  XMLString::release(&message);
  throw 0;

}



// ---------------------------------------------------------------------------
//  Name: XMLFParser::warning
// 
//  Describtion: implements handling of warning exceptions
//
//  Pre:  See xerces docs
//        
//
//  Post: See xerces docs 
//        
//        
//
//
// ---------------------------------------------------------------------------



void XMLFParser::warning(const SAXParseException& e) throw(int)
{
  char* fileName = XMLString::transcode(e.getSystemId());
  char* message=XMLString::transcode(e.getMessage());
  
  formatErrorMsg("Warning at file : ",
		 fileName,
		 e.getLineNumber(),
		 e.getColumnNumber(),
		 message,
		 top->data);

  //Propagate exception higher up in the calling stack
  XMLString::release(&fileName);
  XMLString::release(&message);
  throw 0;
}



// ---------------------------------------------------------------------------
//  XMLFParser: Overrides of the SAX DocumentHandler interface
//              See xerces docs for more info
// ---------------------------------------------------------------------------



// ---------------------------------------------------------------------------
//  Name: XMLFParser::characters
// 
//  Describtion: implements handling of raw data by sax parser
//
//  Pre:  See xerces docs
//        
//
//  Post: See xerces docs 
//        
//        
//
//
// ---------------------------------------------------------------------------


#ifdef __XERCES_2_8__

void XMLFParser::characters(const     XMLCh* const    chars, 
			    const   unsigned int    length)
#else
  
void XMLFParser::characters(const     XMLCh* const    chars, 
			    XMLSize_t length)
#endif
{
  fFormatter->formatBuf(chars, length, XMLFormatter::NoEscapes);
}





// ---------------------------------------------------------------------------
//  Name: XMLFParser::endDocument
// 
//  Describtion: implements action invoked when end of document encountered
//
//  Pre:  See xerces docs
//        
//
//  Post: Document Parse tree is formatted using virtual function Format
//        
//        
//
//
// ---------------------------------------------------------------------------

void XMLFParser::endDocument()
{
}


// ---------------------------------------------------------------------------
//  Name: XMLFParser::endElement
// 
//  Describtion: implements action invoked when end of xml block is encountered
//
//  Pre:  See xerces docs
//        
//
//  Post: Parse tree last node pointer is adjusted
//        
//        
//
//
// ---------------------------------------------------------------------------

void XMLFParser::endElement(const XMLCh* const uri,
				   const XMLCh* const localname,
				   const XMLCh* const qname)
{
  // after block has been parsed move the end pointer in the parse tree
  // to its parent so we can process next child

  endPtr = endPtr->parent;
}



// ---------------------------------------------------------------------------
//  Name: XMLFParser::startDocument
// 
//  Describtion: implements action invoked when start of document encountered
//
//  Pre:  See xerces docs
//        
//
//  Post: 
//        
//        
//
//
// ---------------------------------------------------------------------------


void XMLFParser::startDocument()
{

}

// ---------------------------------------------------------------------------
//  Name: XMLFParser::startElement
// 
//  Describtion: implements action invoked when start of xml block encountered
//
//  Pre:  See xerces docs
//        
//
//  Post: New parse tree node is created, initialized and added to the tree
//        
//        
//
//
// ---------------------------------------------------------------------------

void XMLFParser::startElement(const   XMLCh* const    uri,
			      const   XMLCh* const    localname,
			      const   XMLCh* const    qname,
			      const   Attributes&     attributes)
{

  // add to the parse tree new dtd element with its attributes

  DTDElement newDTDElement;
  char* tmpPtr = XMLString::transcode(qname);
  newDTDElement.name = tmpPtr;
  XMLString::release(&tmpPtr);

  for (int indx = 0; indx < attributes.getLength(); indx++)
    {
      tmpPtr = XMLString::transcode(attributes.getQName(indx));
      newDTDElement.attributes.Append(tmpPtr);
      XMLString::release(&tmpPtr);

      tmpPtr = XMLString::transcode(attributes.getValue(indx));
      newDTDElement.attributes.Append(tmpPtr);
      XMLString::release(&tmpPtr);
    };  

  // move end pointer of parse tree to the latest child

  endPtr->children.Append(newDTDElement);
  endPtr->children.GetLast().parent = endPtr;
  endPtr = & (endPtr->children.GetLast());

}


// ---------------------------------------------------------------------------
//  XMLFParser: Overrides of the SAX entityResolver interface
//              See xerces docs for more info
// ---------------------------------------------------------------------------


// ---------------------------------------------------------------------------
//  Name: XMLFParser::resolveEntity
// 
//  Describtion: allows overwriting of location of DTD encountered in xml doc
//
//  Pre:  See xerces docs
//        
//
//  Post: Returns 0 for using what is found in xml (default action)
//        
//        
//
//
// ---------------------------------------------------------------------------

InputSource* XMLFParser::resolveEntity (const XMLCh* const publicId, const XMLCh* const systemId )
{
  return DTDSource  ;
};

// ---------------------------------------------------------------------------
//  Name: XMLFParser::FormatErroMsg
// 
//  Describtion: Simple message formatting routine
//               
//
//  Pre:  
//
//
//  Post: outMessage contains formatted text
//        
//        
//        
//
// ---------------------------------------------------------------------------



void XMLFParser::formatErrorMsg(const char* inHeader,
				const char* inFilePath,
				int  inLineNo,
				int  inColNo,
				const char* inMessage,
				AhelpStrings& outMessage)
{

  outMessage = inHeader;
  outMessage.append(inFilePath);
  outMessage.append(", line ");
  outMessage.append(inLineNo);
  outMessage.append(", char ");
  outMessage.append(inColNo);
  outMessage.append("\n Message: ");
  outMessage.append(inMessage);
  


};

==> ciao-4.8-src-core.tar.gz: ciao-4.8/src/lib/help/XMLFParser.cc <==
// 
//  Copyright (C) 2002,2005,2007,2009,2014,2015  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//




// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include "cxcconfig.h"
#include "XMLFParser.hh"




int XMLFParser::globalEnvInitFlag = 0;
int XMLFParser::instanceCount = 0;
SAX2XMLReader* XMLFParser::parser;
XMLFormatter* XMLFParser::fFormatter;

// ---------------------------------------------------------------------------
//  Name: XMLFParser::XMLFParser 
// 
//  Describtion: Constructor for XMLFParser
//
//  Pre:  
//        
//
//  Post: Throws int exception if init error occured
//        Because of global data in sax2 parser number of XMLFParser instances
//        is maintained
//        
//
//
// ---------------------------------------------------------------------------

XMLFParser::XMLFParser() throw(int)
{
  instanceCount++;
}


// ---------------------------------------------------------------------------
//  Name: XMLFParser::~XMLFParser 
// 
//  Describtion: Destructor for XMLFParser
//
//  Pre:  
//        
//
//  Post: Because of global data in sax2 parser number of XMLFParser instances
//        is maintained
//        
//
//
// ---------------------------------------------------------------------------

XMLFParser::~XMLFParser()
{

  --instanceCount;

  if (( !instanceCount) && ( globalEnvInitFlag) )
    {
      delete parser;
      delete fFormatter; 
      XMLPlatformUtils::Terminate();
      globalEnvInitFlag=0;

    }

}


// ---------------------------------------------------------------------------
//  Name: XMLFParser::parse 
// 
//  Describtion: the only entry point to the parser
//
//  Pre:  inFilePath - xml file to be parsed
//        inDTDPath  - location of dtd, if set to non empty string it overrides
//                     dtd specified in xml files
//        inValidateFlag - true if validation of xml file for complience with 
//                           dtd should be performed
//  Post: outData - formatted content of xml file
//        on error outData contains error message if any
//        Return values: 1 on success, 0 otherwise
//
//
// ---------------------------------------------------------------------------

int XMLFParser::parse(const AhelpStrings& inFilePath,const AhelpStrings& inDTDPath,bool inValidateFlag,DTDElement& outElement)
{

  int status = 1;
  char* tmpPtr;
  top=&outElement;
  endPtr = top;

  top->Reset();


  // Make sure xax2 environment is initialized only once
  
  if ( !globalEnvInitFlag )
    {
      try
	{
	  XMLPlatformUtils::Initialize();
	  parser = XMLReaderFactory::createXMLReader();
	  
	  // fFormatter is used by sax2 parser to convert text from internal
	  // representation to specified format 
	  
	  fFormatter = new XMLFormatter("UTF-8", this, XMLFormatter::NoEscapes,XMLFormatter::UnRep_CharRef);
	}
      catch (const XMLException& e)
	{
	  outElement.data="XMLFParser::XMLFParser  Error during initialization! :";
	  outElement.data.append("Exception message is:  ");
	  tmpPtr=XMLString::transcode(e.getMessage());
	  outElement.data.append(tmpPtr);
	  XMLString::release(&tmpPtr);
	  status = 0;
	}
      globalEnvInitFlag=1;
    };
  
  if ( status )
    {
      
      try
	{
	  if ( inDTDPath == "")
	    {
	      DTDSource = 0;
	    }
	  else
	    {
	      XMLCh* tmpXMLPtr =  XMLString::transcode(inDTDPath.chars());
	      DTDSource = new LocalFileInputSource(tmpXMLPtr);
	      XMLString::release(&tmpXMLPtr);
	    };
	  
	  if ( inValidateFlag )
	    {      
	      parser->setFeature(XMLUni::fgSAX2CoreValidation, true);
	      parser->setFeature(XMLUni::fgXercesDynamic, false);
	    }
	  else
	    {
	      parser->setFeature(XMLUni::fgSAX2CoreValidation, false);
	    };
	  
	  
	  // sax2 needs handlers for various aspects of parsing
	  // see sax2 documentation in xerces package for more
	  
	  parser->setContentHandler(this);
	  parser->setErrorHandler(this);
	  parser->setEntityResolver(this);
	  
	  parser->parse(inFilePath.chars());

	}
      catch (const XMLException& e)
	{
	  outElement.data = "Error in XMLFParser::parse while parsing '";
	  outElement.data.append(inFilePath.chars() );
	  outElement.data.append("'\n");
	  outElement.data.append("Exception message is:  \n");
	  tmpPtr= XMLString::transcode(e.getMessage());
	  outElement.data.append(tmpPtr);
	  XMLString::release(&tmpPtr);
	  status = 0;
	}
      catch(...)
	{
	  status = 0;
	};
    };


  if ( ! status )
    {
      outElement.children.RemoveAll();
    };

  return status;
};


// ---------------------------------------------------------------------------
//  XMLFParser: Overrides of the output formatter target interface
//              See xerces docs for more info
// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------
//  Name: XMLFParser::writeChars 
// 
//  Describtion: implements handling of converted raw data
//
//  Pre:  
//        
//
//  Post: Adds new raw data element to the parse tree
//        
//        
//
//
// ---------------------------------------------------------------------------



#ifdef __XERCES_2_8__

void XMLFParser::writeChars(const XMLByte* const toWrite,
			    const unsigned int count,
			    XMLFormatter* const formatter)
  
#else
  
void XMLFParser::writeChars(const   XMLByte* const  toWrite, 
			    XMLSize_t    count, 
			    XMLFormatter* const   formatter)

#endif
{

  //Handle new raw data 

  DTDElement newDTDElement;
  int mergeFlag=0;
  char *buf=0;

  //Create blockText out of it


  if ( count > 0 )
    {
      buf = new char[count+1];
      strncpy(buf,(char*)toWrite,count);
      buf[count]=0;
    };


  // Try to merge it with previous raw data block
  if ( top->CanCompact(endPtr->name) && ( count==0))
    {
      mergeFlag = 1;
    }
  else
    {
      
      if ( endPtr->children.GetLength())
	{
	  
	  if ( endPtr->children.GetLast().name == DTDElement::TOK_PCDATA )
	    {
	      endPtr->children.GetLast().data.append(buf);
	      mergeFlag = 1;
	    };
	};
    };
  
  if ( ! mergeFlag )
    {
      newDTDElement.data = buf;
      newDTDElement.name = DTDElement::TOK_PCDATA;
      
      endPtr->children.Append(newDTDElement);
      endPtr->children.GetLast().parent = endPtr;
    };

  if ( buf)
    {
      delete[] buf;
    };

}


// ---------------------------------------------------------------------------
//  XMLFParser: Overrides of the SAX ErrorHandler interface
//              See xerces docs for more info
// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------
//  Name: XMLFParser::error
// 
//  Describtion: implements handling of error exeptions
//
//  Pre:  See xerces docs
//        
//
//  Post: See xerces docs 
//        
//        
//
//
// ---------------------------------------------------------------------------


void XMLFParser::error(const SAXParseException& e) throw(int)
{
  char* fileName = XMLString::transcode(e.getSystemId());
  char* message=XMLString::transcode(e.getMessage());
  formatErrorMsg("Error at file : ",
		 fileName,
		 e.getLineNumber(),
		 e.getColumnNumber(),
		 message,
		 top->data);

  XMLString::release(&fileName);
  XMLString::release(&message);
  //Propagate exception higher up in the calling stack

  throw 0;
}



// ---------------------------------------------------------------------------
//  Name: XMLFParser::error
// 
//  Describtion: implements handling of ftal error exeptions
//
//  Pre:  See xerces docs
//        
//
//  Post: See xerces docs 
//        
//        
//
//
// ---------------------------------------------------------------------------



void XMLFParser::fatalError(const SAXParseException& e) throw(int)
{
  char* fileName=XMLString::transcode(e.getSystemId());
  char* message=XMLString::transcode(e.getMessage());
  formatErrorMsg("Fatal Error at file : ",
		 fileName,
		 e.getLineNumber(),
		 e.getColumnNumber(),
		 message,
		 top->data);

  //Propagate exception higher up in the calling stack
  XMLString::release(&fileName);
  XMLString::release(&message);
  throw 0;

}



// ---------------------------------------------------------------------------
//  Name: XMLFParser::warning
// 
//  Describtion: implements handling of warning exceptions
//
//  Pre:  See xerces docs
//        
//
//  Post: See xerces docs 
//        
//        
//
//
// ---------------------------------------------------------------------------



void XMLFParser::warning(const SAXParseException& e) throw(int)
{
  char* fileName = XMLString::transcode(e.getSystemId());
  char* message=XMLString::transcode(e.getMessage());
  
  formatErrorMsg("Warning at file : ",
		 fileName,
		 e.getLineNumber(),
		 e.getColumnNumber(),
		 message,
		 top->data);

  //Propagate exception higher up in the calling stack
  XMLString::release(&fileName);
  XMLString::release(&message);
  throw 0;
}



// ---------------------------------------------------------------------------
//  XMLFParser: Overrides of the SAX DocumentHandler interface
//              See xerces docs for more info
// ---------------------------------------------------------------------------



// ---------------------------------------------------------------------------
//  Name: XMLFParser::characters
// 
//  Describtion: implements handling of raw data by sax parser
//
//  Pre:  See xerces docs
//        
//
//  Post: See xerces docs 
//        
//        
//
//
// ---------------------------------------------------------------------------


#ifdef __XERCES_2_8__

void XMLFParser::characters(const     XMLCh* const    chars, 
			    const   unsigned int    length)
#else
  
void XMLFParser::characters(const     XMLCh* const    chars, 
			    XMLSize_t length)
#endif
{
  fFormatter->formatBuf(chars, length, XMLFormatter::NoEscapes);
}





// ---------------------------------------------------------------------------
//  Name: XMLFParser::endDocument
// 
//  Describtion: implements action invoked when end of document encountered
//
//  Pre:  See xerces docs
//        
//
//  Post: Document Parse tree is formatted using virtual function Format
//        
//        
//
//
// ---------------------------------------------------------------------------

void XMLFParser::endDocument()
{
}


// ---------------------------------------------------------------------------
//  Name: XMLFParser::endElement
// 
//  Describtion: implements action invoked when end of xml block is encountered
//
//  Pre:  See xerces docs
//        
//
//  Post: Parse tree last node pointer is adjusted
//        
//        
//
//
// ---------------------------------------------------------------------------

void XMLFParser::endElement(const XMLCh* const uri,
				   const XMLCh* const localname,
				   const XMLCh* const qname)
{
  // after block has been parsed move the end pointer in the parse tree
  // to its parent so we can process next child

  endPtr = endPtr->parent;
}



// ---------------------------------------------------------------------------
//  Name: XMLFParser::startDocument
// 
//  Describtion: implements action invoked when start of document encountered
//
//  Pre:  See xerces docs
//        
//
//  Post: 
//        
//        
//
//
// ---------------------------------------------------------------------------


void XMLFParser::startDocument()
{

}

// ---------------------------------------------------------------------------
//  Name: XMLFParser::startElement
// 
//  Describtion: implements action invoked when start of xml block encountered
//
//  Pre:  See xerces docs
//        
//
//  Post: New parse tree node is created, initialized and added to the tree
//        
//        
//
//
// ---------------------------------------------------------------------------

void XMLFParser::startElement(const   XMLCh* const    uri,
			      const   XMLCh* const    localname,
			      const   XMLCh* const    qname,
			      const   Attributes&     attributes)
{

  // add to the parse tree new dtd element with its attributes

  DTDElement newDTDElement;
  char* tmpPtr = XMLString::transcode(qname);
  newDTDElement.name = tmpPtr;
  XMLString::release(&tmpPtr);

  for (unsigned int indx = 0; indx < attributes.getLength(); indx++)
    {
      tmpPtr = XMLString::transcode(attributes.getQName(indx));
      newDTDElement.attributes.Append(tmpPtr);
      XMLString::release(&tmpPtr);

      tmpPtr = XMLString::transcode(attributes.getValue(indx));
      newDTDElement.attributes.Append(tmpPtr);
      XMLString::release(&tmpPtr);
    };  

  // move end pointer of parse tree to the latest child

  endPtr->children.Append(newDTDElement);
  endPtr->children.GetLast().parent = endPtr;
  endPtr = & (endPtr->children.GetLast());

}


// ---------------------------------------------------------------------------
//  XMLFParser: Overrides of the SAX entityResolver interface
//              See xerces docs for more info
// ---------------------------------------------------------------------------


// ---------------------------------------------------------------------------
//  Name: XMLFParser::resolveEntity
// 
//  Describtion: allows overwriting of location of DTD encountered in xml doc
//
//  Pre:  See xerces docs
//        
//
//  Post: Returns 0 for using what is found in xml (default action)
//        
//        
//
//
// ---------------------------------------------------------------------------

InputSource* XMLFParser::resolveEntity (const XMLCh* const publicId, const XMLCh* const systemId )
{
  return DTDSource  ;
};

// ---------------------------------------------------------------------------
//  Name: XMLFParser::FormatErroMsg
// 
//  Describtion: Simple message formatting routine
//               
//
//  Pre:  
//
//
//  Post: outMessage contains formatted text
//        
//        
//        
//
// ---------------------------------------------------------------------------



void XMLFParser::formatErrorMsg(const char* inHeader,
				const char* inFilePath,
				int  inLineNo,
				int  inColNo,
				const char* inMessage,
				AhelpStrings& outMessage)
{

  outMessage = inHeader;
  outMessage.append(inFilePath);
  outMessage.append(", line ");
  outMessage.append(inLineNo);
  outMessage.append(", char ");
  outMessage.append(inColNo);
  outMessage.append("\n Message: ");
  outMessage.append(inMessage);
  


};
